﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using InstrumentDrivers;

/// <summary>
/// Version history:
/// 
/// Version 3.0.0
/// - updated for rsnrpz driver 3.5.6.0
/// - added Reset() and zeroing
/// - changed averaging to auto with defined digits resolution
/// - corrected coercing in Watt->dBm calculation

/// Version 2.0.0
/// - example reworked to simple Console application
/// - updated for rsnrpz driver 3.4.1.0
/// 
/// </summary>

namespace rsnrpz_Continuous_Average_Example
{
    /// <summary>
    /// Simple example of Continuous Average measurement with R&S NRP-Zxx.
    /// rsnrpz64.cs wrapper taken from rsnrpz driver 3.5.6.0 64-bit: "c:\Program Files\IVI Foundation\VISA\Win64\Include\rsnrpz64.cs"
    /// To build the 32-bit version, install the rsnrpz_vxipnp_3_5_6_0_32bit driver and replace the rsnrpz64.cs with its 32-bit equivalent:
    /// "c:\Program Files (x86)\IVI Foundation\VISA\WinNT\include\rsnrpz.cs"
    /// </summary>
    class Program
    {
        static rsnrpz sensor = null;
        static void Main(string[] args)
        {
            int measurementsCount = 10;
            bool complete = false;
            double result = 0.0;
            var resourceName = new StringBuilder(1000);
            var sensorType = new StringBuilder(1000);
            var sensorSN = new StringBuilder(1000);
            int avgResolution = 2; // Range 1 - 4, the higher resolution, the more stable results, but longer measurement time

            try
            {
                // Find the first available sensor
                sensor = new rsnrpz("USB::0xAAD::*?", true, true);
            }
            catch(Exception)
            {
                Console.WriteLine("\n\nError initializing the sensor. Press any key to finish");
                Console.ReadKey();
                return;
            }
            
            try
            {
                sensor.GetSensorInfo(1, resourceName, sensorType, sensorSN); //read sensor info
                Console.WriteLine("Sensor Resource Name: " + resourceName);
                Console.WriteLine("Sensor Type: " + sensorType);
                Console.WriteLine("Sensor Serial Number: " + sensorSN);
                
                Console.Write("\nApplying continuous averaging settings ...");
                sensor.reset();
                sensor.chan_mode(1, rsnrpzConstants.SensorModeContav); //Continuous average mode non-buffered
                sensor.chan_setCorrectionFrequency(1, 100E6); //correction frequency 100MHz
                sensor.avg_configureAvgAuto(1, avgResolution); // auto averaging with defined digits resolution
                sensor.trigger_setSource(1, rsnrpzConstants.TriggerSourceImmediate); //trigger immediate
                sensor.chan_setContAvSmoothingEnabled(1, false); //smoothing of the results using a low-pass filter
                Console.WriteLine(" finished.");

                Console.Write("\nZeroing sensor, remove any signal from its input ...");
                sensor.chan_zero(1);
                //wait until the zeroing is ready with the timeout of 10 seconds
                System.DateTime tout = System.DateTime.Now.AddSeconds(10.0);
                do
                {
                    sensor.chan_isZeroComplete(1, out complete);
                    System.Threading.Thread.Sleep(500);
                } while (!complete && (System.DateTime.Now.CompareTo(tout) < 0));
                Console.WriteLine(" finished.");

                Console.Write("\nApply signal to the sensor input and press any key to continue ...");
                Console.ReadKey();

                Console.WriteLine("\n\nMeasured values in dBm:");
                for (int i = 0; i < measurementsCount; i++)
                {
                    //start the measurement
                    sensor.chans_initiate();

                    //wait until the measurement is ready with the timeout of 3 seconds
                    tout = System.DateTime.Now.AddSeconds(3.0);
                    do
                    {
                        sensor.chan_isMeasurementComplete(1, out complete);
                        System.Threading.Thread.Sleep(1);
                    } while (!complete && (System.DateTime.Now.CompareTo(tout) < 0));

                    if (complete)
                    {
                        sensor.meass_fetchMeasurement(1, out result); //fetch the result in Watts
                        //Watt to dBm, for low and sometimes slightly negative values coerce to -200dBm
                        double measValue = result > 0 ? (10 * Math.Log(result) / Math.Log(10)) + 30.0 : -200;
                        Console.WriteLine(measValue.ToString("F3"));
                    }
                }
            }

            catch (Exception ex)
            {
                Console.WriteLine("\nException occured: " + ex.Message);
            }
            finally
            {
                sensor.Dispose();

                Console.WriteLine("\n\nPress any key to finish");
                Console.ReadKey();
            }
        }
    }
}
